//--------------------------------------------------------------------------
//
//  Copyright 2009, PlanetPSoC.com.
//  Author: M. Ganesh Raaja
//
//	Permission is hereby granted, free of charge, to any person or organization
//	obtaining a copy of the software and accompanying documentation covered by
//	this license (the "Software") to use, reproduce, display, distribute,
//	execute, and transmit the Software, and to prepare derivative works of the
//	Software, and to permit third-parties to whom the Software is furnished to
//	do so, all subject to the following:
//	
//	The copyright notices in the Software and this entire statement, including
//	the above license grant, this restriction and the following disclaimer,
//	must be included in all copies of the Software, in whole or in part, and
//	all derivative works of the Software, unless such copies or derivative
//	works are solely in the form of machine-executable object code generated by
//	a source language processor.
//	
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
//	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
//	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
//	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//	DEALINGS IN THE SOFTWARE.
//--------------------------------------------------------------------------
//	Project: ADC_LCD_UART 
//
//	This project demonstrates scanning three analog channels using the Delta Sigma
//	ADC and displaying the results on an LCD and also over an UART
//	
//	The project measures the analog input signals present on P0_0, P0_1 and P0_2 using
//	a Delta Sigma ADC configured for 16 bit resolution.  The hex results of each of the
//	channels is displayed on the LCD.  A string with the Channel number (X, Y, Z), the
//	hex ADC value and the calculated voltage on the channel is sent over an UART
//	component configured at 19200 baud rate.  An Analog Mux is used to switch between the
//	input signals.  The ADC is configured for Fast FIR mode.  Everytime the input is changed
//	two samples are dropped and only the third sample is taken as the valid sample.  This
//	is required for a Delta Sigma converter.

//	For constructing the serial string, the function strcat is used.  This function 
//	concatenates the source string with the destination string.
//
//--------------------------------------------------------------------------

#include <device.h>
#include <stdio.h>
#include <string.h>

// Array to store the 16 bit ADC result
unsigned int ADCResult[3];

// Array to store the calculated voltage
float Vin[3];

// Function prototypes
void ScanChannels(void);
void PrintResults(void);

void main()
{
	
	/* Place your initialization/startup code here (e.g. MyInst_Start()) */
	ADC_Start();
	ADC_StartConvert();

	LCD_Start();
    UART_1_Start();
	AMux_1_Start();
	Opamp_1_Start();
	//PGA_1_Start();
    CYGlobalIntEnable; /* Uncomment this line to enable global interrupts. */
    for(;;)
    {
		CyDelay(50);
		ScanChannels();
		PrintResults();
    }
}

// Function that scans all the channels and stores the results in the ADCResult and Vin arrays
void ScanChannels(void)
{
    uint8 Channel;
	uint8 i;

	// Scan three channels
	for(Channel=0; Channel < 3; Channel++)
	{
		// Change to next channel
		AMux_1_Select(Channel);
		
		// In this loop, we do three ADC measurements.  After switching channel, the first two results
		// should be dropped.  When we exit from this loop, the ADCResult will have the third sample which
		// is a valid sample
		for(i=0; i<3; i++)
		{
			ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
			ADCResult[Channel] = ADC_GetResult16();
			if (ADC_WAIT_FOR_RESULT)
			ADC_StopConvert();
			else
			{
			}
		}
		
		// Calculate Voltage
		Vin[Channel] = ADC_CountsTo_Volts(ADCResult[Channel]);
	}

}

//Function that prints the hex ADC Count values on the LCD and the ADC count & Voltage values to UART
void PrintResults(void)
{
char OutputString[10];
char SerialOutString[64];

	// Clear the output string
	strcpy(SerialOutString,  "");

	// Convert X axis into Hex and display on LCD
	sprintf(OutputString, "%5.3f", Vin[0]);
	LCD_Position(0,0);
	LCD_PrintString("    ");
	LCD_Position(0,0);
	LCD_PrintString(OutputString);
	
	// Add X Axis value to the serial out string
	strcat(SerialOutString, "Input 1 : ");
	strcat(SerialOutString, OutputString);

	// Convert X axis voltage into ascii and add to serial out string
	//sprintf(OutputString, "%5.3f", Vin[0]);
	//strcat(SerialOutString, " ");
	//strcat(SerialOutString, OutputString);

	// Convert Y axis into Hex and display on LCD
	sprintf(OutputString, "%5.3f", Vin[1]);
	LCD_Position(0,8);
	LCD_PrintString("    ");
	LCD_Position(0,8);
	LCD_PrintString(OutputString);

	// Add Y Axis value to the serial out string
	strcat(SerialOutString, "    ; Input 2: ");
	strcat(SerialOutString, OutputString);

	// Convert Y axis voltage into ascii and add to serial out string
	//sprintf(OutputString, "%5.3f", Vin[1]);
	//strcat(SerialOutString, " ");
	//strcat(SerialOutString, OutputString);

	// Convert Z axis into Hex and display on LCD
	sprintf(OutputString, "%5.3f", Vin[2]);
	LCD_Position(1,0);
	LCD_PrintString("    ");
	LCD_Position(1,0);
	LCD_PrintString(OutputString);

	// Add Z Axis value to the serial out string
	strcat(SerialOutString, "   ;Input 3 : ");
	strcat(SerialOutString, OutputString);

	// Convert Z axis voltage into ascii and add to serial out string
	//sprintf(OutputString, "%5.3f", Vin[2]);
	//strcat(SerialOutString, " ");
	//strcat(SerialOutString, OutputString);

	// Print the String to Serial port and terminate with CR and LF
	UART_1_PutString(SerialOutString);
	UART_1_PutCRLF(0x20);
}

/* [] END OF FILE */

